@@ -29,7 +29,7 @@
 #include <string.h>
 #include <strings.h> /* strcasecmp */
 
-#include "parse_string.h"
+#include <parse_string.h>
 #include <libdvbv5/dvb-log.h>
 #include <libdvbv5/dvb-fe.h>
 
@@ -300,12 +300,12 @@ static struct charset_conv en300468_latin_00_to_utf8[256] = {
 	[0xff] = { 2, {0xc2, 0xad, } },
 };
 
-void iconv_to_charset(struct dvb_v5_fe_parms *parms,
-		      char *dest,
-		      size_t destlen,
-		      const unsigned char *src,
-		      size_t len,
-		      char *type, char *output_charset)
+void dvb_iconv_to_charset(struct dvb_v5_fe_parms *parms,
+			  char *dest,
+			  size_t destlen,
+			  const unsigned char *src,
+			  size_t len,
+			  char *input_charset, char *output_charset)
 {
 	char out_cs[strlen(output_charset) + 1 + sizeof(CS_OPTIONS)];
 	char *p = dest;
@@ -313,12 +313,12 @@ void iconv_to_charset(struct dvb_v5_fe_parms *parms,
 	strcpy(out_cs, output_charset);
 	strcat(out_cs, CS_OPTIONS);
 
-	iconv_t cd = iconv_open(out_cs, type);
+	iconv_t cd = iconv_open(out_cs, input_charset);
 	if (cd == (iconv_t)(-1)) {
 		memcpy(p, src, len);
 		p[len] = '\0';
 		dvb_logerr("Conversion from %s to %s not supported\n",
-				type, output_charset);
+				input_charset, output_charset);
 	} else {
 		iconv(cd, (ICONV_CONST char **)&src, &len, &p, &destlen);
 		iconv_close(cd);
@@ -327,14 +327,13 @@ void iconv_to_charset(struct dvb_v5_fe_parms *parms,
 }
 
 static void charset_conversion(struct dvb_v5_fe_parms *parms, char **dest, const unsigned char *s,
-			       size_t len,
-			       char *type, char *output_charset)
+			       size_t len, char *input_charset)
 {
 	size_t destlen = len * 3;
 	int need_conversion = 1;
 
 	/* Special handler for ISO-6937 */
-	if (!strcasecmp(type, "ISO-6937")) {
+	if (!strcasecmp(input_charset, "ISO-6937")) {
 		char *p = *dest;
 		unsigned char *tmp;
 		unsigned char *p1, *p2;
@@ -348,12 +347,12 @@ static void charset_conversion(struct dvb_v5_fe_parms *parms, char **dest, const
 		*p = '\0';
 
 		/* If desired charset is not UTF-8, prepare for conversion */
-		if (strcasecmp(output_charset, "UTF-8")) {
+		if (strcasecmp(parms->output_charset, "UTF-8")) {
 			tmp = (unsigned char *)*dest;
 			len = p - *dest;
 
 			*dest = malloc(destlen + 1);
-			type = "UTF-8";
+			input_charset = "UTF-8";
 			s = tmp;
 		} else
 			need_conversion = 0;
@@ -362,16 +361,16 @@ static void charset_conversion(struct dvb_v5_fe_parms *parms, char **dest, const
 
 	/* Convert from original charset to the desired one */
 	if (need_conversion)
-		iconv_to_charset(parms, *dest, destlen, s, len, type,
-				 output_charset);
+		dvb_iconv_to_charset(parms, *dest, destlen, s, len,
+				     input_charset,
+				     parms->output_charset);
 }
 
-void parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
-		  const unsigned char *src, size_t len,
-		  char *default_charset, char *output_charset)
+void dvb_parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
+		      const unsigned char *src, size_t len)
 {
 	size_t destlen, i, len2 = 0;
-	char *p, *p2, *type = default_charset;
+	char *p, *p2, *type = parms->default_charset;
 	unsigned char *tmp1 = NULL, *tmp2 = NULL;
 	const unsigned char *s;
 	int emphasis = 0;
@@ -400,7 +399,7 @@ void parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
 		case 0x09:	type = "ISO-8859-13";		break;
 		case 0x0a:	type = "ISO-8859-14";		break;
 		case 0x0b:	type = "ISO-8859-15";		break;
-		case 0x11:	type = "ISO-10646";		break;
+		case 0x11:	type = "ISO-10646/UCS2";	break;
 		case 0x12:	type = "ISO-2022-KR";		break;
 		case 0x13:	type = "GB2312";		break;
 		case 0x14:	type = "UTF-16BE";		break;
@@ -442,7 +441,7 @@ void parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
 	*emph = malloc(destlen + 1);
 
 	/* Remove special chars */
-	if (!strncasecmp(type, "ISO-8859", 8) || !strcasecmp(type, "ISO-6937")) {
+	if (!strncasecmp(type, "ISO-8859", 8) || !strcasecmp(type, "ISO-6937") || !strcasecmp(type, "ISO-10646/UTF-8")) {
 		/*
 		 * Handles the ISO/IEC 10646 1-byte control codes
 		 * (EN 300 468 v1.11.1 Table A.1)
@@ -467,12 +466,41 @@ void parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
 		*p2 = '\0';
 		len = p - (char *)tmp1;
 		len2 = p2 - (char *)tmp2;
-	} else {
-		dvb_logerr("charset %s not implemented", type);
+	} else if (!strcasecmp(type, "ISO-10646/UCS2")) {
 		/*
-		 * FIXME: need to handle the ISO/IEC 10646 2-byte control codes
+		 * Handles the ISO/IEC 10646 2-bytes control codes
 		 * (EN 300 468 v1.11.1 Table A.2)
 		 */
+		uint16_t *in_code = (void *)src;
+		uint16_t *out_code;
+		uint16_t *out_emph;
+
+		tmp1 = malloc(len + 2);
+		tmp2 = malloc(len + 2);
+		out_code = (void *)tmp1;
+		out_emph = (void *)tmp2;
+
+		for (i = 0; i < len / 2; i ++, in_code++) {
+			uint16_t code = *in_code;
+
+			/*
+			 * FIXME: should it do bswap16(code) here?
+			 */
+			if (code == 0xe086)
+				emphasis = 1;
+			else if (code == 0xe087 && emphasis)
+				emphasis = 0;
+			else if (code >= 0xe080 && code <= 0xe09f)
+				continue;
+
+			*out_code++ = code;
+			if (emphasis)
+				*out_emph++ = code;
+		}
+		*out_code = 0;
+		*out_emph = 0;
+		len = (char *)out_code - (char *)tmp1;
+		len2 = (char *)out_emph - (char *)tmp2;
 	}
 
 	if (tmp1)
@@ -480,7 +508,7 @@ void parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
 	else
 		s = src;
 
-	charset_conversion(parms, dest, s, len, type, output_charset);
+	charset_conversion(parms, dest, s, len, type);
 	/* The code had over-sized the space. Fix it. */
 	if (*dest)
 		*dest = realloc(*dest, strlen(*dest) + 1);
@@ -493,7 +521,7 @@ void parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
 		free (*emph);
 		*emph = NULL;
 	} else {
-		charset_conversion(parms, emph, tmp2, len2, type, output_charset);
+		charset_conversion(parms, emph, tmp2, len2, type);
 		*emph = realloc(*emph, strlen(*emph) + 1);
 	}
 
