@@ -273,7 +273,7 @@ static const char *asus_board_vendor[] = {
 	NULL };
 
 static const char *asus_board_name[] = {
-	"A3[A-Z]*", "A7M",
+	"A3[A-Z]*", "A6R*", "A7M", "A7Sn",
 	"B50[A-Z]*",
 	"F[23579][A-Z]*", "F70[A-Z]*", "F[58]2[A-Z]*",
 	"G[12][A-Z]*", "G[57]0[A-Z]*",
